from .abstract import AbstractTransform


class DoNothingTransform(AbstractTransform):
    def __init__(self, grad: bool = False, **kwargs):
        """
        Forward input

        Parameters
        ----------
        grad:
            enable gradient computation inside transformation
        kwargs:
            keyword arguments passed to superclass
        """
        super().__init__(grad=grad, **kwargs)

    def forward(self, **data) -> dict:
        """
        Forward input

        Parameters
        ----------
        data: dict
            input dict

        Returns
        -------
        dict
            input dict
        """
        return data



import torch
import random
from typing import Union, Sequence

from .abstract import BaseTransform, PerSampleTransform, AbstractTransform, PerChannelTransform
from .functional.intensity import *


class ClampTransform(BaseTransform):
    def __init__(self, min: float, max: float, keys: Sequence = ('data',), grad: bool = False, **kwargs):
        """
        Apply augment_fn to keys

        Parameters
        ----------
        augment_fn: callable
            function for augmentation
        dims: tuple
            axes which should be mirrored
        grad: bool
            enable gradient computation inside transformation
        kwargs:
            keyword arguments passed to augment_fn
        """
        super().__init__(augment_fn=torch.clamp, keys=keys, grad=grad, min=min, max=max, **kwargs)


class NormRangeTransform(PerSampleTransform):
    def __init__(self, min: float, max: float, keys: Sequence = ('data',), per_channel: bool = True,
                 grad: bool = False, **kwargs):
        """
        Scale data to provided min and max values

        Parameters
        ----------
        min: float
            minimal value
        max: float
            maximal value
        keys: Sequence
            keys to normalize
        per_channel: bool
            normalize per channel
        grad: bool
            enable gradient computation inside transformation
        kwargs:
            keyword arguments passed to normalization function
        """
        super().__init__(augment_fn=norm_range, keys=keys, grad=grad,
                         min=min, max=max, per_channel=per_channel, **kwargs)


class NormMinMaxTransform(PerSampleTransform):
    def __init__(self, keys: Sequence = ('data',), per_channel: bool = True, grad: bool = False, **kwargs):
        """
        Scale data to [0, 1]

        Parameters
        ----------
        keys: Sequence
            keys to normalize
        per_channel: bool
            normalize per channel
        grad: bool
            enable gradient computation inside transformation
        kwargs:
            keyword arguments passed to normalization function
        """
        super().__init__(augment_fn=norm_min_max, keys=keys, grad=grad,
                         per_channel=per_channel, **kwargs)


class NormZeroMeanUnitStdTransform(PerSampleTransform):
    def __init__(self, keys: Sequence = ('data',), per_channel: bool = True, grad: bool = False, **kwargs):
        """
        Normalize mean to zero and std to one

        Parameters
        ----------
        keys: Sequence
            keys to normalize
        per_channel: bool
            normalize per channel
        grad: bool
            enable gradient computation inside transformation
        kwargs:
            keyword arguments passed to normalization function
        """
        super().__init__(augment_fn=norm_zero_mean_unit_std, keys=keys, grad=grad,
                         per_channel=per_channel, **kwargs)


class NormMeanStdTransform(PerSampleTransform):
    def __init__(self, mean: Union[float, Sequence], std: Union[float, Sequence],
                 keys: Sequence = ('data',), per_channel: bool = True, grad: bool = False, **kwargs):
        """
        Normalize mean and std with provided values

        Parameters
        ----------
        mean: float or Sequence
            used for mean normalization
        std: float or Sequence
            used for std normalization
        keys: Sequence
            keys to normalize
        per_channel: bool
            normalize per channel
        grad: bool
            enable gradient computation inside transformation
        kwargs:
            keyword arguments passed to normalization function
        """
        super().__init__(augment_fn=norm_mean_std, keys=keys, grad=grad,
                         mean=mean, std=std, per_channel=per_channel, **kwargs)


class NoiseTransform(PerChannelTransform):
    def __init__(self, noise_type: str, per_channel: bool = False,
                 keys: Sequence = ('data',), grad: bool = False, **kwargs):
        """
        Add noise to data

        Parameters
        ----------
        noise_type: str
            supports all inplace functions of a pytorch tensor
        per_channel: bool
            enable transformation per channel
        keys: Sequence
            keys to normalize
        grad: bool
            enable gradient computation inside transformation
        kwargs:
            keyword arguments passed to noise function

        See Also
        --------
        :func:`torch.Tensor.normal_`, :func:`torch.Tensor.exponential_`
        """
        super().__init__(augment_fn=add_noise, per_channel=per_channel, keys=keys,
                         grad=grad, noise_type=noise_type, **kwargs)


class ExponentialNoiseTransform(NoiseTransform):
    def __init__(self, lambd: float, keys: Sequence = ('data',), grad: bool = False, **kwargs):
        """
        Add exponential noise to data

        Parameters
        ----------
        lambd: float
            lambda of exponential distribution
        keys: Sequence
            keys to normalize
        grad: bool
            enable gradient computation inside transformation
        kwargs:
            keyword arguments passed to noise function
        """
        super().__init__(noise_type='exponential_', lambd=lambd, keys=keys, grad=grad, **kwargs)


class GaussianNoiseTransform(NoiseTransform):
    def __init__(self, mean: float, std: float, keys: Sequence = ('data',), grad: bool = False, **kwargs):
        """
        Add noise to data

        Parameters
        ----------
        mean: float
            mean of normal distribution
        std: float
            std of normal distribution
        keys: Sequence
            keys to normalize
        grad: bool
            enable gradient computation inside transformation
        kwargs:
            keyword arguments passed to noise function
        """
        super().__init__(noise_type='normal_', mean=mean, std=std, keys=keys, grad=grad, **kwargs)


class GammaCorrectionTransform(AbstractTransform):
    def __init__(self, gamma: Union[float, Sequence] = (0.5, 2),
                 keys: Sequence = ('data',), grad: bool = False, **kwargs):
        """
        Apply gamma correction as augmentation

        Parameters
        ----------
        gamma: float or sequence
            if gamma is float it is always applied. if gamma is a sequence it is interpreted as
            the minimal and maximal value. If the maximal value is greater than one, the transform
            chooses gamma <1 in 50% of the cases and gamma >1 in the other cases.
        keys: Sequence
            keys to normalize
        grad: bool
            enable gradient computation inside transformation
        kwargs:
            keyword arguments passed to superclass
        """
        super().__init__(augment_fn=gamma_correction, keys=keys, grad=grad)
        self.kwargs = kwargs
        self.gamma = gamma
        if not check_scalar(self.gamma):
            if not len(self.gamma) == 2:
                raise TypeError(f"Gamma needs to be scalar or a Sequence with two entries "
                                f"(min, max), found {self.gamma}")

    def forward(self, **data) -> dict:
        """
        Apply transformation

        Parameters
        ----------
        data: dict
            dict with tensors

        Returns
        -------
        dict
            dict with augmented data
        """
        if check_scalar(self.gamma):
            _gamma = self.gamma
        elif self.gamma[1] < 1:
            _gamma = random.uniform(self.gamma[0], self.gamma[1])
        else:
            if random.random() < 0.5:
                _gamma = _gamma = random.uniform(self.gamma[0], 1)
            else:
                _gamma = _gamma = random.uniform(1, self.gamma[1])

        for _key in self.keys:
            data[_key] = self.augment_fn(data[_key], _gamma, **self.kwargs)
        return data


class RandomValuePerChannelTransform(PerChannelTransform):
    def __init__(self, augment_fn: callable, random_mode: str, random_kwargs: dict = None,
                 per_channel: bool = False, keys: Sequence = ('data',),
                 grad: bool = False, **kwargs):
        """
        Apply augmentations which take random values as input by keyword
        :param:`value`

        Parameters
        ----------
        random_mode: str
            specifies distribution which should be used to sample additive value (supports all
            random generators from python random package)
        random_kwargs: dict
            additional arguments for random function
        per_channel: bool
            enable transformation per channel
        keys: Sequence
            keys which should be augmented
        grad: bool
            enable gradient computation inside transformation
        kwargs:
            keyword arguments passed to augment_fn
        """
        super().__init__(augment_fn=augment_fn, per_channel=per_channel,
                         keys=keys, grad=grad, **kwargs)
        self.random_mode = random_mode
        self.random_kwargs = {} if random_kwargs is None else random_kwargs

    def forward(self, **data) -> dict:
        """
        Perform Augmentation.

        Parameters
        ----------
        data: dict
            dict with data

        Returns
        -------
        dict
            augmented data
        """
        if self.per_channel:
            random_seed = random.random()
            for _key in self.keys:
                random.seed(random_seed)
                out = torch.empty_like(data[_key])
                for _i in range(data[_key].shape[1]):
                    rand_value = self.random_fn(**self.random_kwargs)
                    out[:, _i] = self.augment_fn(data[_key][:, _i], value=rand_value,
                                                 out=out[:, _i], **self.kwargs)
                data[_key] = out
            return data
        else:
            self.kwargs["value"] = self.random_fn(**self.random_kwargs)
            return super().forward(**data)

    @property
    def random_mode(self) -> str:
        """
        Get random mode

        Returns
        -------
        str
            random mode
        """
        return self._random_mode

    @random_mode.setter
    def random_mode(self, mode) -> None:
        """
        Set random mode

        Parameters
        ----------
        mode: str
            specifies distribution which should be used to sample additive value (supports all
            random generators from python random package)
        """
        self._random_mode = mode
        self.random_fn = getattr(random, mode)


class RandomAddValue(RandomValuePerChannelTransform):
    def __init__(self, random_mode: str, random_kwargs: dict = None, per_channel: bool = False,
                 keys: Sequence = ('data',), grad: bool = False, **kwargs):
        """
        Increase values additively

        Parameters
        ----------
        random_mode: str
            specifies distribution which should be used to sample additive value (supports all
            random generators from python random package)
        random_kwargs: dict
            additional arguments for random function
        per_channel: bool
            enable transformation per channel
        keys: Sequence
            keys which should be augmented
        grad: bool
            enable gradient computation inside transformation
        kwargs:
            keyword arguments passed to augment_fn
        """
        super().__init__(augment_fn=add_value, random_mode=random_mode,
                         random_kwargs=random_kwargs, per_channel=per_channel,
                         keys=keys, grad=grad, **kwargs)


class RandomScaleValue(RandomValuePerChannelTransform):
    def __init__(self, random_mode, random_kwargs: dict = None, per_channel: bool = False,
                 keys: Sequence = ('data',), grad: bool = False, **kwargs):
        """
        Scale values

        Parameters
        ----------
        random_mode: str
            specifies distribution which should be used to sample additive value (supports all
            random generators from python random package)
        random_kwargs: dict
            additional arguments for random function
        per_channel: bool
            enable transformation per channel
        keys: Sequence
            keys which should be augmented
        grad: bool
            enable gradient computation inside transformation
        kwargs:
            keyword arguments passed to augment_fn
        """
        super().__init__(augment_fn=scale_by_value, random_mode=random_mode,
                         random_kwargs=random_kwargs, per_channel=per_channel,
                         keys=keys, grad=grad, **kwargs)



import torch
import random
from .abstract import RandomDimsTransform, AbstractTransform
from typing import Union, Sequence
from itertools import permutations

from .functional.spatial import mirror, rot90


class MirrorTransform(RandomDimsTransform):
    def __init__(self, dims: Sequence, keys: Sequence = ('data',),
                 prob: Union[float, Sequence] = 0.5, grad: bool = False, **kwargs):
        """
        Random mirror transform

        Parameters
        ----------
        dims: tuple
            axes which should be mirrored
        keys: tuple
            keys which should be mirrored
        prob: typing.Union[float, tuple]
            probability for mirror. If float value is provided, it is used
            for all dims
        grad: bool
            enable gradient computation inside transformation
        kwargs:
            keyword arguments passed to superclass
        """
        super().__init__(augment_fn=mirror, dims=dims, keys=keys, prob=prob, grad=grad, **kwargs)


class Rot90Transform(AbstractTransform):
    def __init__(self, dims: tuple, keys: tuple = ('data',),
                 prob: Union[float, Sequence] = 0.5, grad: bool = False, **kwargs):
        """
        Randomly rotate 90 degree around dims

        Parameters
        ----------
        dims: tuple
            dims which should be rotated. If more than two dims are provided,
            two dimensions are randomly chosen at each call
        keys: tuple
            keys which should be rotated
        prob: typing.Union[float, tuple]
            probability for rotation
        grad: bool
            enable gradient computation inside transformation
        kwargs:
            keyword arguments passed to superclass

        See Also
        --------
        :func:`torch.Tensor.rot90`
        """
        super().__init__(grad=grad, **kwargs)
        self.dims = dims
        self.keys = keys
        self.prob = prob

    def forward(self, **data) -> dict:
        """
        Apply transformation

        Parameters
        ----------
        data: dict
            dict with tensors

        Returns
        -------
        dict
            dict with augmented data
        """
        if torch.rand(1) < self.prob:
            k = random.randint(0, 4)
            rand_dims = self._permutations[random.randint(0, len(self._permutations))]

            for key in self.keys:
                data[key] = rot90(data[key], k=k, dims=rand_dims)
        return data

    @property
    def dims(self):
        return self._dims

    @dims.setter
    def dims(self, dims):
        self._dims = dims
        self._permutations = tuple(permutations(dims, 2))



import unittest
from unittest.mock import Mock, call
import torch

from rising.transforms.abstract import *


class AddTransform(AbstractTransform):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.grad_tensor = torch.rand(1, 1, 32, 32, requires_grad=True)

    def forward(self, **data) -> dict:
        data["data"] = data["data"] + self.grad_tensor
        return data


def sum_dim(data, dims, **kwargs):
    if dims:
        dims = [d + 2 for d in dims]
        return torch.sum(data, dims, **kwargs)
    else:
        return data


class TestAbstractTransform(unittest.TestCase):
    def setUp(self) -> None:
        torch.manual_seed(0)
        self.batch_dict = {
            "data": torch.rand(1, 1, 32, 32),
            "seg": torch.rand(1, 1, 32, 32),
            "label": torch.arange(3)
        }

    def test_abstract_transform(self):
        trafo = AbstractTransform(grad=False, internal0=True)
        self.assertTrue(trafo.internal0)
        with self.assertRaises(NotImplementedError):
            trafo(**self.batch_dict)

    def test_abstract_transform_no_grad(self):
        trafo = AddTransform(grad=False)
        output = trafo(**self.batch_dict)

        self.assertIsNone(output["data"]._grad_fn)
        with self.assertRaises(RuntimeError):
            output["data"].mean().backward()

    def test_abstract_transform_grad(self):
        trafo = AddTransform(grad=True)
        output = trafo(**self.batch_dict)
        self.assertIsNotNone(output["data"]._grad_fn)

        output["data"].mean().backward()
        self.assertIsNotNone(trafo.grad_tensor._grad)

    def test_base_transform(self):
        trafo = BaseTransform(lambda x: x + 50, keys=('data', 'seg'))
        output = trafo(**self.batch_dict)

        diff_data = (output["data"] - self.batch_dict["data"]).mean().item()
        diff_seg = (output["seg"] - self.batch_dict["seg"]).mean().item()
        diff_label = (output["label"] - self.batch_dict["label"]).float().mean().item()

        self.assertEqual(diff_data, 50)
        self.assertEqual(diff_seg, 50)
        self.assertEqual(diff_label, 0)

    def test_per_sample_transform(self):
        mock = Mock(return_value=0)

        def augment_fn(inp, *args, **kwargs):
            return mock(inp)

        trafo = PerSampleTransform(augment_fn, keys=('label',))
        output = trafo(**self.batch_dict)
        calls = [call(torch.tensor([0])), call(torch.tensor([1])),
                 call(torch.tensor([2])), ]
        mock.assert_has_calls(calls)

    def test_per_channel_transform_per_channel_true(self):
        mock = Mock(return_value=0)

        def augment_fn(inp, *args, **kwargs):
            return mock(inp)

        trafo = PerChannelTransform(augment_fn, per_channel=True, keys=('label',))
        self.batch_dict["label"] = self.batch_dict["label"][None]
        output = trafo(**self.batch_dict)
        calls = [call(torch.tensor([0])), call(torch.tensor([1])),
                 call(torch.tensor([2])), ]
        mock.assert_has_calls(calls)

    def test_per_channel_transform_per_channel_false(self):
        mock = Mock(return_value=0)

        def augment_fn(inp, *args, **kwargs):
            return mock(inp)

        trafo = PerChannelTransform(augment_fn, per_channel=False, keys=('label',))
        self.batch_dict["label"] = self.batch_dict["label"][None]
        output = trafo(**self.batch_dict)
        mock.assert_called_once()

    def test_random_dims_transform(self):
        torch.manual_seed(0)
        self.batch_dict["data"] = torch.rand(1, 1, 32, 16)
        trafo = RandomDimsTransform(sum_dim, dims=(0, 1))
        shapes = [trafo(**self.batch_dict)["data"].shape for i in range(4)]
        self.assertEqual(shapes[0], torch.Size([1, 1]))
        self.assertEqual(shapes[1], torch.Size([1, 1, 32, 16]))
        self.assertEqual(shapes[2], torch.Size([1, 1, 16]))
        self.assertEqual(shapes[3], torch.Size([1, 1, 32]))


if __name__ == '__main__':
    unittest.main()



import unittest
import torch
from rising.transforms.utility import *


class TestUtilityTransforms(unittest.TestCase):
    def setUp(self) -> None:
        torch.manual_seed(0)

    def test_do_nothing_transform(self):
        inp = {"data": 0, "seg": 1}
        trafo = DoNothingTransform()
        outp = trafo(**inp)
        self.assertEqual(outp["data"], 0)
        self.assertEqual(outp["seg"], 1)


if __name__ == '__main__':
    unittest.main()



import torch
import unittest
import numpy as np

from rising.ops import torch_one_hot, np_one_hot


class TestOneHot(unittest.TestCase):
    def test_torch_one_hot(self):
        target = torch.tensor([0, 1, 2])
        target_onehot = torch_one_hot(target, 3)
        expected_onehot = torch.tensor([[1, 0, 0], [0, 1, 0], [0, 0, 1]])
        self.assertTrue((expected_onehot == target_onehot).all())

    def test_np_one_hot(self):
        target = np.array([0, 1, 2])
        target_onehot = np_one_hot(target, 3)
        expected_onehot = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]])
        self.assertTrue((expected_onehot == target_onehot).all())


if __name__ == '__main__':
    unittest.main()



import torch
import numpy as np


def torch_one_hot(target: torch.Tensor, num_classes: int) -> torch.Tensor:
    """
    Compute one hot encoding of input tensor

    Parameters
    ----------
    target: torch.Tensor
        tensor to be converted
    num_classes: int
        number of classes

    Returns
    -------
    torch.Tensor
        one hot encoded tensor
    """
    dtype, device = target.dtype, target.device
    target_onehot = torch.zeros(*target.shape, num_classes,
                                dtype=dtype, device=device)
    target_onehot.scatter_(1, target.unsqueeze(1), 1.0)
    return target_onehot.to(dtype=dtype, device=device)


def np_one_hot(target: np.ndarray, num_classes: int) -> np.ndarray:
    """
    Compute one hot encoding of input array

    Parameters
    ----------
    target: np.ndarray
        array to be converted
    num_classes: int
        number of classes

    Returns
    -------
    np.ndarray
        one hot encoded array
    """
    dtype = target.dtype
    target_onehot = np.zeros((*target.shape, num_classes), dtype=dtype)
    for c in range(num_classes):
        target_onehot[..., c] = target == c
    return target_onehot



import torch
import typing
from typing import Callable, Union, Sequence, Any

from rising.utils import check_scalar


augment_callable = Callable[[torch.Tensor], Any]
augment_axis_callable = Callable[[torch.Tensor, Union[float, Sequence]], Any]


class AbstractTransform(torch.nn.Module):
    def __init__(self, grad: bool = False, **kwargs):
        """
        Base class for all transforms

        Parameters
        ----------
        grad: bool
            enable gradient computation inside transformation
        """
        super().__init__()
        self.grad = grad
        for key, item in kwargs.items():
            setattr(self, key, item)

    def __call__(self, *args, **kwargs) -> typing.Any:
        """
        Call super class with correct torch context

        Parameters
        ----------
        args:
            forwarded positional arguments
        kwargs:
            forwarded keyword arguments

        Returns
        -------

        """
        if self.grad:
            context = torch.enable_grad()
        else:
            context = torch.no_grad()

        with context:
            return super().__call__(*args, **kwargs)

    def forward(self, **data) -> dict:
        """
        Implement transform functionality here

        Parameters
        ----------
        data: dict
            dict with data

        Returns
        -------
        dict
            dict with transformed data
        """
        raise NotImplementedError


class BaseTransform(AbstractTransform):
    def __init__(self, augment_fn: augment_callable, keys: Sequence = ('data',),
                 grad: bool = False, **kwargs):
        """
        Apply augment_fn to keys

        Parameters
        ----------
        augment_fn: callable
            function for augmentation
        keys: Sequence
            keys which should be augmented
        grad: bool
            enable gradient computation inside transformation
        kwargs:
            keyword arguments passed to augment_fn
        """
        super().__init__(grad=grad)
        self.augment_fn = augment_fn
        self.keys = keys
        self.kwargs = kwargs

    def forward(self, **data) -> dict:
        """
        Apply transformation

        Parameters
        ----------
        data: dict
            dict with tensors

        Returns
        -------
        dict
            dict with augmented data
        """
        for _key in self.keys:
            data[_key] = self.augment_fn(data[_key], **self.kwargs)
        return data


class PerSampleTransform(BaseTransform):
    def forward(self, **data) -> dict:
        """
        Apply transformation to each sample in batch individually
        :param:`augment_fn` must be callable with option :param:`out`
        where results are saved in

        Parameters
        ----------
        data: dict
            dict with tensors

        Returns
        -------
        dict
            dict with augmented data
        """
        for _key in self.keys:
            out = torch.empty_like(data[_key])
            for _i in range(data[_key].shape[0]):
                out[_i] = self.augment_fn(data[_key][_i], out=out[_i], **self.kwargs)
            data[_key] = out
        return data


class PerChannelTransform(BaseTransform):
    def __init__(self, augment_fn: augment_callable, per_channel: bool = False,
                 keys: Sequence = ('data',), grad: bool = False, **kwargs):
        """
        Apply transformation per channel (but still to whole batch)

        Parameters
        ----------
        augment_fn: callable
            function for augmentation
        per_channel: bool
            enable transformation per channel
        keys: Sequence
            keys which should be augmented
        grad: bool
            enable gradient computation inside transformation
        kwargs:
            keyword arguments passed to augment_fn
        """
        super().__init__(augment_fn=augment_fn, keys=keys, grad=grad, **kwargs)
        self.per_channel = per_channel

    def forward(self, **data) -> dict:
        """
        Apply transformation

        Parameters
        ----------
        data: dict
            dict with tensors

        Returns
        -------
        dict
            dict with augmented data
        """
        if self.per_channel:
            for _key in self.keys:
                out = torch.empty_like(data[_key])
                for _i in range(data[_key].shape[1]):
                    out[:, _i] = self.augment_fn(data[_key][:, _i], out=out[:, _i], **self.kwargs)
                data[_key] = out
            return data
        else:
            return super().forward(**data)


class RandomDimsTransform(AbstractTransform):
    def __init__(self, augment_fn: augment_axis_callable, dims: Sequence, keys: Sequence = ('data',),
                 prob: Union[float, Sequence] = 0.5, grad: bool = False, **kwargs):
        """
        Randomly choose axes to perform augmentation on

        Parameters
        ----------
        augment_fn: callable
            function for augmentation
        dims: tuple
            possible axes
        keys: tuple
            keys which should be augmented
        prob: typing.Union[float, tuple]
            probability for mirror. If float value is provided, it is used
            for all dims
        grad: bool
            enable gradient computation inside transformation
        kwargs:
            keyword arguments passed to augment_fn
        """
        super().__init__(grad=grad)
        self.augment_fn = augment_fn
        self.dims = dims
        self.keys = keys
        if check_scalar(prob):
            prob = (prob,) * len(dims)
        self.prob = prob
        self.kwargs = kwargs

    def forward(self, **data) -> dict:
        """
        Apply transformation

        Parameters
        ----------
        data: dict
            dict with tensors

        Returns
        -------
        dict
            dict with augmented data
        """
        rand_val = torch.rand(len(self.dims), requires_grad=False)
        dims = [_dim for _dim, _prob in zip(self.dims, self.prob) if rand_val[_dim] < _prob]

        for key in self.keys:
            data[key] = self.augment_fn(data[key], dims=dims, **self.kwargs)
        return data



from .abstract import AbstractTransform


class MapToSeq(AbstractTransform):
    def __init__(self, *keys, grad: bool = False, **kwargs):
        """
        Convert dict to sequence

        Parameters
        ----------
        keys: tuple
            keys which are mapped into sequence.
        grad: bool
            enable gradient computation inside transformation
        kwargs:
            additional keyword arguments passed to superclass
        """
        super().__init__(grad=grad, **kwargs)
        self.keys = keys

    def forward(self, **data) -> tuple:
        """
        Convert input

        Parameters
        ----------
        data: dict
            input dict

        Returns
        -------
        tuple
            mapped data
        """
        return tuple(data[_k] for _k in self.keys)


class SeqToMap(AbstractTransform):
    def __init__(self, *keys, grad: bool = False, **kwargs):
        """
        Convert sequence to dict

        Parameters
        ----------
        keys: tuple
            keys which are mapped into dict.
        grad: bool
            enable gradient computation inside transformation
        kwargs:
            additional keyword arguments passed to superclass
        """
        super().__init__(grad=grad, **kwargs)
        self.keys = keys

    def forward(self, *data) -> dict:
        """
        Convert input

        Parameters
        ----------
        data: tuple
            input tuple

        Returns
        -------
        dict
            mapped data
        """
        return {_key: data[_idx] for _idx, _key in enumerate(self.keys)}



import unittest
from math import isclose

from rising.transforms.functional.intensity import *


class TestIntensityFunctional(unittest.TestCase):
    def setUp(self) -> None:
        self.batch_2d = torch.rand(3, 3)[None, None]

    def test_norm_range(self):
        inp = (self.batch_2d[0] * 10) + 1
        outp = norm_range(inp, 2, 3, per_channel=False)

        self.assertEqual(outp.min().item(), 2)
        self.assertEqual(outp.max().item(), 3)

    def test_norm_range_per_channel(self):
        inp = (self.batch_2d[0] * 10) + 1
        outp = norm_range(inp, 2, 3, per_channel=True)

        for c in range(inp.shape[0]):
            self.assertEqual(outp[c].min().item(), 2)
            self.assertEqual(outp[c].max().item(), 3)

    def test_norm_min_max(self):
        inp = (self.batch_2d[0] * 10) + 1
        outp = norm_min_max(inp, per_channel=False)

        self.assertEqual(outp.min().item(), 0)
        self.assertEqual(outp.max().item(), 1)

    def test_norm_min_max_per_channels(self):
        inp = (self.batch_2d[0] * 10) + 1
        outp = norm_min_max(inp, per_channel=True)

        for c in range(inp.shape[0]):
            self.assertEqual(outp[c].min().item(), 0)
            self.assertEqual(outp[c].max().item(), 1)

    def test_zero_mean_unit_std(self):
        inp = (self.batch_2d[0] * 10) + 1
        outp = norm_zero_mean_unit_std(inp, per_channel=False)

        self.assertTrue(isclose(outp.mean().item(), 0, abs_tol=1e-06))
        self.assertTrue(isclose(outp.std().item(), 1, rel_tol=1e-06))

    def test_zero_mean_unit_std_per_channel(self):
        inp = (self.batch_2d[0] * 10) + 1
        outp = norm_zero_mean_unit_std(inp, per_channel=True)

        for c in range(inp.shape[0]):
            self.assertTrue(isclose(outp[c].mean().item(), 0, abs_tol=1e-06))
            self.assertTrue(isclose(outp[c].std().item(), 1, rel_tol=1e-06))

    def test_mean_std(self):
        inp = (self.batch_2d[0] * 10) + 1
        outp = norm_mean_std(inp, inp.mean().item(), inp.std().item(), per_channel=False)

        self.assertTrue(isclose(outp.mean().item(), 0, abs_tol=1e-06))
        self.assertTrue(isclose(outp.std().item(), 1, rel_tol=1e-06))

    def test_mean_std_per_channel(self):
        inp = (self.batch_2d[0] * 10) + 1
        channel_mean = [inp[c].mean().item() for c in range(inp.shape[0])]
        channel_std = [inp[c].std().item() for c in range(inp.shape[0])]
        outp = norm_mean_std(inp, channel_mean, channel_std, per_channel=True)

        for c in range(inp.shape[0]):
            self.assertTrue(isclose(outp[c].mean().item(), 0, abs_tol=1e-06))
            self.assertTrue(isclose(outp[c].std().item(), 1, rel_tol=1e-06))

    def test_mean_std_per_channel_scalar(self):
        # TEST: add error sensitive test to check correct behavior
        inp = (self.batch_2d[0] * 10) + 1
        outp = norm_mean_std(inp, inp.mean().item(), inp.std().item(), per_channel=True)

    def test_add_noise(self):
        outp = add_noise(self.batch_2d.clone(), 'normal', mean=75, std=1)
        diff = (outp - self.batch_2d).abs().mean()
        self.assertTrue(diff > 50)

    def test_gamma_correction(self):
        outp = gamma_correction(self.batch_2d, 2)
        self.assertTrue((self.batch_2d.pow(2) == outp).all())

    def test_add_value(self):
        outp = add_value(self.batch_2d, 2)
        self.assertTrue((torch.add(self.batch_2d, 2) == outp).all())

    def test_scale_by_value(self):
        outp = scale_by_value(self.batch_2d, 2)
        self.assertTrue((torch.mul(self.batch_2d, 2) == outp).all())


if __name__ == '__main__':
    unittest.main()



import torch
from rising.transforms.abstract import AbstractTransform


def chech_data_preservation(trafo: AbstractTransform, batch: dict, key: str = 'data') -> bool:
    """
    Checks for inplace modification of input data

    Parameters
    ----------
    trafo: AbstractTransform
        transforamtion
    batch: dict
        batch with torch.Tensor data
    key: str
        key to examine

    Returns
    -------
    bool
        true if data did not change inplace
    """
    orig = batch[key].clone()
    _ = trafo(**batch)
    return (orig == batch[key]).all()



import torch
from typing import Sequence, Union

from rising.utils import check_scalar


def mirror(data: torch.Tensor, dims: Union[int, Sequence[int]]) -> torch.Tensor:
    """
    Mirror data at dims

    Parameters
    ----------
    data: torch.Tensor
        input data
    dims: int or Sequence[int]
        dimensions to mirror

    Returns
    -------
    torch.Tensor
        tensor with mirrored dimensions
    """
    if check_scalar(dims):
        dims = (dims,)
    dims = [d + 2 for d in dims]
    return data.flip(dims)


def rot90(data: torch.Tensor, k: int, dims: Union[int, Sequence[int]]):
    """
    Rotate 90 degrees around dims

    Parameters
    ----------
    data: torch.Tensor
        input data
    k: int
        number of times to rotate
    dims: int or Sequence[int]
        dimensions to mirror

    Returns
    -------
    torch.Tensor
        tensor with mirrored dimensions
    """
    dims = [d + 2 for d in dims]
    return torch.rot90(data, k, dims)



import torch

from typing import Union, Sequence

from rising.utils import check_scalar


def norm_range(data: torch.Tensor, min: float, max: float,
               per_channel: bool = True, out: torch.Tensor = None) -> torch.Tensor:
    """
    Scale range of tensor

    Parameters
    ----------
    data: torch.Tensor
        input data. Per channel option supports [C,H,W] and [C,H,W,D].
    min: float
        minimal value
    max: float
        maximal value
    per_channel: bool
        range is normalized per channel
    out: torch.Tensor
        if provided, result is saved in here

    Returns
    -------
    torch.Tensor
        normalized data
    """
    if out is None:
        out = torch.zeros_like(data)

    out = norm_min_max(data, per_channel=per_channel, out=out)
    _range = max - min
    out = (out * _range) + min
    return out


def norm_min_max(data: torch.Tensor, per_channel: bool = True,
                 out: torch.Tensor = None) -> torch.Tensor:
    """
    Scale range to [0,1]

    Parameters
    ----------
    data: torch.Tensor
        input data. Per channel option supports [C,H,W] and [C,H,W,D].
    per_channel: bool
        range is normalized per channel
    out: torch.Tensor
        if provided, result is saved in here

    Returns
    -------
    torch.Tensor
        scaled data
    """
    if out is None:
        out = torch.zeros_like(data)

    if per_channel:
        for _c in range(data.shape[0]):
            _min = data[_c].min()
            _range = data[_c].max() - _min
            out[_c] = (data[_c] - _min) / _range
    else:
        _min = data.min()
        _range = data.max() - _min
        out = (data - _min) / _range

    return out


def norm_zero_mean_unit_std(data: torch.Tensor, per_channel: bool = True,
                            out: torch.Tensor = None) -> torch.Tensor:
    """
    Normalize mean to zero and std to one

    Parameters
    ----------
    data: torch.Tensor
        input data. Per channel option supports [C,H,W] and [C,H,W,D].
    per_channel: bool
        range is normalized per channel
    out: torch.Tensor
        if provided, result is saved in here

    Returns
    -------
    torch.Tensor
        normalized data
    """
    if out is None:
        out = torch.zeros_like(data)

    if per_channel:
        for _c in range(data.shape[0]):
            out[_c] = (data[_c] - data[_c].mean()) / data[_c].std()
    else:
        out = (data - data.mean()) / data.std()

    return out


def norm_mean_std(data: torch.Tensor, mean: Union[float, Sequence], std: Union[float, Sequence],
                  per_channel: bool = True, out: torch.Tensor = None) -> torch.Tensor:
    """
    Normalize mean and std with provided values

    Parameters
    ----------
    data: torch.Tensor
        input data. Per channel option supports [C,H,W] and [C,H,W,D].
    mean: float or Sequence
        used for mean normalization
    std: float or Sequence
        used for std normalization
    per_channel: bool
        range is normalized per channel
    out: torch.Tensor
        if provided, result is saved into out

    Returns
    -------
    torch.Tensor
        normalized data
    """
    if out is None:
        out = torch.zeros_like(data)

    if per_channel:
        if check_scalar(mean):
            mean = [mean] * data.shape[0]
        if check_scalar(std):
            std = [std] * data.shape[0]
        for _c in range(data.shape[0]):
            out[_c] = (data[_c] - mean[_c]) / std[_c]
    else:
        out = (data - mean) / std

    return out


def add_noise(data: torch.Tensor, noise_type: str, out: torch.Tensor = None,
              **kwargs) -> torch.Tensor:
    """
    Add noise to input

    Parameters
    ----------
    data: torch.Tensor
        input data
    noise_type: str
        supports all inplace functions of a pytorch tensor
    out: torch.Tensor
        if provided, result is saved in here
    kwargs:
        keyword arguments passed to generating function

    Returns
    -------
    torch.Tensor
        data with added noise

    See Also
    --------
    :func:`torch.Tensor.normal_`, :func:`torch.Tensor.exponential_`
    """
    if not noise_type.endswith('_'):
        noise_type = noise_type + '_'
    noise_tensor = torch.empty_like(data, requires_grad=False)
    getattr(noise_tensor, noise_type)(**kwargs)
    return torch.add(data, noise_tensor, out=out)


def gamma_correction(data: torch.Tensor, gamma: float) -> torch.Tensor:
    """
    Apply gamma correction to data
    (currently this functions is intended as an interface in case
    additional functionality should be added to transform)

    Parameters
    ----------
    data: torch.Tensor
        input data
    gamma: float
        gamma for correction

    Returns
    -------
    torch.Tensor
    """
    return data.pow(gamma)


def add_value(data: torch.Tensor, value: float, out: torch.Tensor = None) -> torch.Tensor:
    """
    Increase brightness additively by value
    (currently this functions is intended as an interface in case
    additional functionality should be added to transform)

    Parameters
    ----------
    data: torch.Tensor
        input data
    value: float
        additive value
    out: torch.Tensor
        if provided, result is saved in here

    Returns
    -------
    torch.Tensor
        augmented data
    """
    return torch.add(data, value, out=out)


def scale_by_value(data: torch.Tensor, value: float, out: torch.Tensor = None) -> torch.Tensor:
    """
    Increase brightness scaled by value
    (currently this functions is intended as an interface in case
    additional functionality should be added to transform)

    Parameters
    ----------
    data: torch.Tensor
        input data
    value: float
        scaling value
    out: torch.Tensor
        if provided, result is saved in here

    Returns
    -------
    torch.Tensor
        augmented data
    """
    return torch.mul(data, value, out=out)



import math
import torch
from typing import Sequence, Union, Callable

from .abstract import AbstractTransform
from rising.utils import check_scalar


class KernelTransform(AbstractTransform):
    def __init__(self, in_channels: int, kernel_size: Union[int, Sequence], dim: int = 2,
                 stride: Union[int, Sequence] = 1, padding: Union[int, Sequence] = 0,
                 padding_mode: str = 'zero', keys: Sequence = ('data',), grad: bool = False,
                 **kwargs):
        """
        Baseclass for kernel based transformations (kernel is applied to each channel individually)

        Parameters
        ----------
        in_channels: int
            number of input channels
        kernel_size: int or sequence
            size of kernel
        dim: int
            number of spatial dimensions
        stride: int or sequence
            stride of convolution
        padding: int or sequence
            padding size for input
        padding_mode: str
            padding mode for input. Supports all modes from :func:`torch.functional.pad` except
            `circular`
        keys: sequence
            keys which should be augmented
        grad: bool
            enable gradient computation inside transformation
        kwargs:
            keyword arguments passed to superclass

        See Also
        --------
        :func:`torch.functional.pad`
        """
        super().__init__(grad=grad, **kwargs)
        self.in_channels = in_channels

        if check_scalar(kernel_size):
            kernel_size = [kernel_size] * dim
        self.kernel_size = kernel_size

        if check_scalar(stride):
            stride = [stride] * dim
        self.stride = stride

        if check_scalar(padding):
            padding = [padding] * dim * 2
        self.padding = padding

        self.padding_mode = padding_mode
        self.keys = keys

        kernel = self.create_kernel()
        self.register_buffer('weight', kernel)
        self.groups = in_channels
        self.conv = self.get_conv(dim)

    @staticmethod
    def get_conv(dim) -> Callable:
        """
        Select convolution with regard to dimension

        Parameters
        ----------
        dim: int
            spatial dimension of data
        """
        if dim == 1:
            return torch.nn.functional.conv1d
        elif dim == 2:
            return torch.nn.functional.conv2d
        elif dim == 3:
            return torch.nn.functional.conv3d
        else:
            raise TypeError('Only 1, 2 and 3 dimensions are supported. Received {}.'.format(dim))

    def create_kernel(self) -> torch.Tensor:
        """
        Create kernel for convolution
        """
        raise NotImplementedError

    def forward(self, **data) -> dict:
        """
        Apply kernel to selected keys

        Parameters
        ----------
        data: dict
            dict with input data

        Returns
        -------
        dict
            dict with transformed data
        """
        for key in self.keys:
            inp_pad = torch.nn.functional.pad(data[key], self.padding, mode=self.padding_mode)
            data[key] = self.conv(inp_pad, weight=self.weight, groups=self.groups, stride=self.stride)
        return data


class GaussianSmoothingTransform(KernelTransform):
    def __init__(self, in_channels: int, kernel_size: Union[int, Sequence],
                 std: Union[int, Sequence], dim: int = 2,
                 stride: Union[int, Sequence] = 1, padding: Union[int, Sequence] = 0,
                 padding_mode: str = 'reflect', keys: Sequence = ('data',), grad: bool = False,
                 **kwargs):
        """
        Perform Gaussian Smoothing.
        Filtering is performed seperately for each channel in the input using a depthwise convolution.
        This code is adapted from: ('https://discuss.pytorch.org/t/is-there-anyway-to-do-gaussian'
        '-filtering-for-an-image-2d-3d-in-pytorch/12351/10')

        Parameters
        ----------
        in_channels: int
            number of input channels
        kernel_size: int or sequence
            size of kernel
        std: int or sequence
            standard deviation of gaussian
        dim: int
            number of spatial dimensions
        stride: int or sequence
            stride of convolution
        padding: int or sequence
            padding size for input
        padding_mode: str
            padding mode for input. Supports all modes from :func:`torch.functional.pad` except
            `circular`
        keys: sequence
            keys which should be augmented
        grad: bool
            enable gradient computation inside transformation
        kwargs:
            keyword arguments passed to superclass

        See Also
        --------
        :func:`torch.functional.pad`
        """
        if check_scalar(std):
            std = [std] * dim
        self.std = std
        super().__init__(in_channels=in_channels, kernel_size=kernel_size, dim=dim, stride=stride,
                         padding=padding, padding_mode=padding_mode, keys=keys, grad=grad, **kwargs)

    def create_kernel(self) -> torch.Tensor:
        """
        Create gaussian blur kernel
        """
        # The gaussian kernel is the product of the gaussian function of each dimension.
        kernel = 1
        meshgrids = torch.meshgrid([
            torch.arange(size, dtype=torch.float32)
            for size in self.kernel_size
        ])

        for size, std, mgrid in zip(self.kernel_size, self.std, meshgrids):
            mean = (size - 1) / 2
            kernel *= 1 / (std * math.sqrt(2 * math.pi)) * torch.exp(-((mgrid - mean) / std) ** 2 / 2)

        # Make sure sum of values in gaussian kernel equals 1.
        kernel = kernel / kernel.sum()

        # Reshape to depthwise convolutional weight
        kernel = kernel.view(1, 1, *kernel.size())
        kernel = kernel.repeat(self.in_channels, *[1] * (kernel.dim() - 1))
        return kernel



import torch
import random
import unittest

from tests.test_transforms import chech_data_preservation
from rising.transforms.spatial import *


class TestSpatialTransforms(unittest.TestCase):
    def setUp(self) -> None:
        torch.manual_seed(0)
        random.seed(0)
        self.batch_dict = {
            "data": torch.arange(1, 10).reshape(1, 1, 3, 3),
            "seg": torch.rand(1, 1, 3, 3),
            "label": torch.arange(3)
        }

    def test_mirror_transform(self):
        trafo = MirrorTransform((0, 1), prob=1)
        outp = trafo(**self.batch_dict)

        self.assertTrue((outp["data"][0, 0] == torch.tensor([[9, 8, 7], [6, 5, 4], [3, 2, 1]])).all())
        self.assertTrue(chech_data_preservation(trafo, self.batch_dict))

        trafo = MirrorTransform((0, 1), prob=0)
        data_orig = self.batch_dict["data"].clone()
        outp = trafo(**self.batch_dict)
        self.assertTrue((outp["data"] == data_orig).all())

    def test_rot90_transform(self):
        random.seed(0)
        trafo = Rot90Transform((0, 1), prob=1)
        outp = trafo(**self.batch_dict)
        self.assertTrue((outp["data"][0, 0] == torch.tensor([[3, 6, 9], [2, 5, 8], [1, 4, 7]])).all())
        self.assertEqual(trafo.dims, (0, 1))
        self.assertTrue(chech_data_preservation(trafo, self.batch_dict))

        trafo = Rot90Transform((0, 1), prob=0)
        data_orig = self.batch_dict["data"].clone()
        outp = trafo(**self.batch_dict)
        self.assertTrue((outp["data"] == data_orig).all())


if __name__ == '__main__':
    unittest.main()



import unittest
import torch
from rising.transforms.format import MapToSeq, SeqToMap


class TestFormat(unittest.TestCase):
    def setUp(self) -> None:
        torch.manual_seed(0)

    def test_map_to_seq(self):
        trafo = MapToSeq('data', 'seg', 'label')
        out = trafo(**{'data': 0, 'seg': 1, 'label': 2})
        self.assertEqual(out[0], 0)
        self.assertEqual(out[1], 1)
        self.assertEqual(out[2], 2)

    def test_seq_to_map(self):
        trafo = SeqToMap('data', 'seg', 'label')
        out = trafo(0, 1, 2)
        self.assertEqual(out['data'], 0)
        self.assertEqual(out['seg'], 1)
        self.assertEqual(out['label'], 2)


if __name__ == '__main__':
    unittest.main()



import torch
import unittest

from rising.transforms.functional.spatial import mirror, rot90


class TestSpatialFunctional(unittest.TestCase):
    def setUp(self) -> None:
        self.batch_2d = torch.arange(1, 10).reshape(3, 3)[None, None]

    def test_mirror_dim0(self):
        inp = self.batch_2d.clone()
        outp = mirror(inp, 0)
        expected = torch.tensor([[7, 8, 9], [4, 5, 6], [1, 2, 3]])
        self.assertTrue((outp == expected).all())

    def test_mirror_dim1(self):
        inp = self.batch_2d.clone()
        outp = mirror(inp, 1)
        expected = torch.tensor([[3, 2, 1], [6, 5, 4], [9, 8, 7]])
        self.assertTrue((outp == expected).all())

    def test_rot90(self):
        inp = self.batch_2d.clone()
        outp = rot90(inp, 1, (0, 1))
        expected = torch.tensor([[3, 6, 9], [2, 5, 8], [1, 4, 7]])
        self.assertTrue((outp == expected).all())


if __name__ == '__main__':
    unittest.main()



import unittest
import torch
import random
from math import isclose
from unittest.mock import Mock, call

from tests.test_transforms import chech_data_preservation
from rising.transforms.intensity import *


class MyTestCase(unittest.TestCase):
    def setUp(self) -> None:
        torch.manual_seed(0)
        random.seed(0)
        self.batch_dict = {
            "data": torch.arange(1, 10).reshape(1, 1, 3, 3).float(),
            "seg": torch.rand(1, 1, 3, 3),
            "label": torch.arange(3)
        }

    def test_clamp_transform(self):
        trafo = ClampTransform(0, 1)

        self.assertTrue(chech_data_preservation(trafo, self.batch_dict))

        outp = trafo(**self.batch_dict)
        self.assertTrue((outp["data"] == torch.ones_like(outp["data"])).all())

    def test_norm_range_transform(self):
        trafo = NormRangeTransform(0.1, 0.2, per_channel=False)
        self.assertTrue(chech_data_preservation(trafo, self.batch_dict))

        trafo = NormRangeTransform(0.1, 0.2, per_channel=True)
        self.assertTrue(chech_data_preservation(trafo, self.batch_dict))

        outp = trafo(**self.batch_dict)
        self.assertTrue(isclose(outp["data"].min().item(), 0.1, abs_tol=1e-6))
        self.assertTrue(isclose(outp["data"].max().item(), 0.2, abs_tol=1e-6))

    def test_norm_min_max_transform(self):
        trafo = NormMinMaxTransform(per_channel=False)
        self.assertTrue(chech_data_preservation(trafo, self.batch_dict))

        trafo = NormMinMaxTransform(per_channel=True)
        self.assertTrue(chech_data_preservation(trafo, self.batch_dict))

        outp = trafo(**self.batch_dict)
        self.assertTrue(isclose(outp["data"].min().item(), 0., abs_tol=1e-6))
        self.assertTrue(isclose(outp["data"].max().item(), 1., abs_tol=1e-6))

    def test_norm_zero_mean_transform(self):
        trafo = NormZeroMeanUnitStdTransform(per_channel=False)
        self.assertTrue(chech_data_preservation(trafo, self.batch_dict))

        trafo = NormZeroMeanUnitStdTransform(per_channel=True)
        self.assertTrue(chech_data_preservation(trafo, self.batch_dict))

        outp = trafo(**self.batch_dict)
        self.assertTrue(isclose(outp["data"].mean().item(), 0., abs_tol=1e-6))
        self.assertTrue(isclose(outp["data"].std().item(), 1., abs_tol=1e-6))

    def test_norm_std_transform(self):
        mean = self.batch_dict["data"].mean().item()
        std = self.batch_dict["data"].std().item()
        trafo = NormMeanStdTransform(mean=mean, std=std, per_channel=False)
        self.assertTrue(chech_data_preservation(trafo, self.batch_dict))

        trafo = NormMeanStdTransform(mean=mean, std=std, per_channel=True)
        self.assertTrue(chech_data_preservation(trafo, self.batch_dict))

        outp = trafo(**self.batch_dict)
        self.assertTrue(isclose(outp["data"].mean().item(), 0., abs_tol=1e-6))
        self.assertTrue(isclose(outp["data"].std().item(), 1., abs_tol=1e-6))

    def test_noise_transform(self):
        trafo = NoiseTransform('normal', mean=75, std=1)
        self.assertTrue(chech_data_preservation(trafo, self.batch_dict))
        self.check_noise_distance(trafo)

    def test_expoential_noise_transform(self):
        trafo = ExponentialNoiseTransform(lambd=0.0001)
        self.assertTrue(chech_data_preservation(trafo, self.batch_dict))
        self.check_noise_distance(trafo)

    def test_gaussian_noise_transform(self):
        trafo = GaussianNoiseTransform(mean=75, std=1)
        self.assertTrue(chech_data_preservation(trafo, self.batch_dict))
        self.check_noise_distance(trafo)

    def check_noise_distance(self, trafo, min_diff=50):
        outp = trafo(**self.batch_dict)
        comp_diff = (outp["data"] - self.batch_dict["data"]).mean().item()
        self.assertTrue(comp_diff > min_diff)

    def test_per_channel_transform_per_channel_true(self):
        mock = Mock(return_value=0)

        def augment_fn(inp, *args, **kwargs):
            return mock(inp)

        trafo = RandomValuePerChannelTransform(
            augment_fn, random_mode="random", per_channel=True, keys=('label',))
        self.batch_dict["label"] = self.batch_dict["label"][None]
        output = trafo(**self.batch_dict)
        calls = [call(torch.tensor([0])), call(torch.tensor([1])),
                 call(torch.tensor([2])), ]
        mock.assert_has_calls(calls)

    def test_random_add_value(self):
        trafo = RandomAddValue("random")
        self.assertTrue(chech_data_preservation(trafo, self.batch_dict))

        random.seed(0)
        rand_val = random.random()
        random.seed(0)
        outp = trafo(**self.batch_dict)
        expected_out = self.batch_dict["data"] + rand_val
        self.assertTrue((outp["data"] == expected_out).all())
        self.assertEqual(trafo.random_mode, "random")

    def test_random_scale_value(self):
        trafo = RandomScaleValue("random")
        self.assertTrue(chech_data_preservation(trafo, self.batch_dict))

        random.seed(0)
        rand_val = random.random()
        random.seed(0)
        outp = trafo(**self.batch_dict)
        expected_out = self.batch_dict["data"] * rand_val
        self.assertTrue((outp["data"] == expected_out).all())
        self.assertEqual(trafo.random_mode, "random")

    def test_gamma_transform_scalar(self):
        trafo = GammaCorrectionTransform(gamma=2)
        self.assertTrue(chech_data_preservation(trafo, self.batch_dict))

        trafo = GammaCorrectionTransform(gamma=2)
        outp = trafo(**self.batch_dict)
        expected_out = self.batch_dict["data"].pow(2)
        self.assertTrue((outp["data"] == expected_out).all())

    def test_gamma_transform_error(self):
        with self.assertRaises(TypeError):
            trafo = GammaCorrectionTransform(gamma=(1, 1, 1))

    def test_gamma_transform_max_smaller_one(self):
        trafo = GammaCorrectionTransform(gamma=(0, 0.9))
        random.seed(0)
        rand_val = random.uniform(0, 0.9)
        random.seed(0)
        outp = trafo(**self.batch_dict)
        expected_out = self.batch_dict["data"].pow(rand_val)
        self.assertTrue((outp["data"] == expected_out).all())

    def test_gamma_transform_max_greater_one(self):
        # (1., 1.) allows switching cases but forces gamma to 1.
        trafo = GammaCorrectionTransform(gamma=(1., 1.))
        random.seed(0)
        for _ in range(5):
            outp = trafo(**self.batch_dict)
            self.assertTrue((outp["data"] == self.batch_dict["data"]).all())


if __name__ == '__main__':
    unittest.main()



import torch
import random
import unittest


from rising.transforms.kernel import *


# TODO: Test: !!!!Implement sensitive tests!!!!

class TestKernelTransforms(unittest.TestCase):
    def setUp(self) -> None:
        torch.manual_seed(0)
        random.seed(0)
        self.batch_dict = {
            "data": torch.zeros(1, 1, 3, 3).float(),
            "seg": torch.rand(1, 1, 3, 3),
            "label": torch.arange(3)
        }

    def test_kernel_transform_get_conv(self):
        conv = KernelTransform.get_conv(1)
        self.assertEqual(conv, torch.nn.functional.conv1d)

        conv = KernelTransform.get_conv(2)
        self.assertEqual(conv, torch.nn.functional.conv2d)

        conv = KernelTransform.get_conv(3)
        self.assertEqual(conv, torch.nn.functional.conv3d)

        with self.assertRaises(TypeError):
            conv = KernelTransform.get_conv(4)

    def test_kernel_transform_error(self):
        with self.assertRaises(NotImplementedError):
            trafo = KernelTransform(in_channels=1, kernel_size=3, std=1,
                                    dim=2, stride=1, padding=1)

    def test_gaussian_smoothing_transform(self):
        trafo = GaussianSmoothingTransform(in_channels=1, kernel_size=3, std=1,
                                           dim=2, stride=1, padding=1)
        self.batch_dict["data"][0, 0, 1] = 1
        outp = trafo(**self.batch_dict)
        print(outp["data"].shape)


if __name__ == '__main__':
    unittest.main()
